REM Dividir objetos con mscara 
REM
REM Para usar con Photo-Paint 8.
REM
REM Este guin divide los objetos seleccionados usando la mscara 
REM actual. La divisin se realiza duplicando cada objeto usando 
REM el comando Cortar objeto (por mscara) en cada copia, y usando 
REM la mscara con uno y la mscara invertida con el otro.
REM 
REM
REM Los objetos pueden dividirse igualmente se mantenga o no  
REM la orden del objeto; esto ltimo es ms rpido. En ambos casos
REM la parte del objeto dentro de la mscara recibir la orden 
REM antes de la parte de fuera de la mscara.
REM
REM Creado el 26 de Octubre, 1995 porRennie Houtman
REM Actualizado para la Versin 8, Octubre 1997
REM ********************************************************

REM Function declarations.

Declare Function RectanglesIntersect(nL1&, nT1&, nR1&, nB1&, nL2&, nT2&, nR2&, nB2&) as Boolean

REM Constants.

Const MAX_OBJECTS = 100
Const CANCELLED = 2
Const TO_FRONT = 0
Const TO_BACK = 1
Const FORWARD = 2
Const BACK = 3
Const strDialogTitle = "Dividir objetos con mscara"

REM Variable delarations.

Dim nObjectCount			as long
Dim nObjectIndex			as long
Dim nCopyIndex				as long
Dim nIndex						as long
Dim nObjectsToSplit		as long
Dim nL								as long
Dim nT								as long
Dim nR								as long
Dim nB								as long
Dim nML								as long
Dim nMT								as long
Dim nMR								as long
Dim nMB								as long
Dim nLeft							as long
Dim nTop							as long
Dim bFoundSelectedObjects as Boolean
Dim anObjectIndices(1 to MAX_OBJECTS) as long

REM Options dialog definition.

BEGIN DIALOG SplitOptionsDialog 165, 65, "strDialogTitle"
	CANCELBUTTON  109, 45, 50, 13
	OKBUTTON  54, 45, 50, 13
	CHECKBOX  10, 5, 107, 11, "Preservar la orden del objeto", MaintainOrder%
	CHECKBOX  10, 19, 148, 11, "Ocultar documento para mejorar velocidad", HideDoc%
END DIALOG

REM Start of the code.
label1 = "Este guin dividir los objetos seleccionados usando la mscara actual."
BEGIN DIALOG Dialog1 196, 74, "Divisor de objetos (Object Splitter)"
	GROUPBOX  2, 1, 191, 53
	TEXT  8, 12, 178, 37, label1
	OKBUTTON  109, 57, 40, 14
	CANCELBUTTON  153, 57, 40, 14
END DIALOG

ret = DIALOG(Dialog1)
' If Cancel is selected, stop the script
IF ret = 2 THEN STOP

WITHOBJECT "CorelPhotoPaint.Automation.8"

	REM Check for a mask.

	if .GetMaskPresent() = False then
		MessageBox "Se necesita una mscara para este guin.", strDialogTitle, 0
		STOP
	endif

	REM Check for objects.

	nObjectCount = .GetObjectCount()
	
	if nObjectCount <= 0 then
		MessageBox "La imagen no contiene objetos",strDialogTitle, 0
		STOP
	endif

	REM Count and save indices of selected objects that intersect
	REM the mask bounding rectangle.

	nObjectsToSplit = 0
	bFoundSelectedObjects = FALSE
	.GetMaskRectangle nML, nMT, nMR, nMB

	for nObjectIndex = 1 to nObjectCount
		if .GetObjectisSelected(nObjectIndex) then
			bFoundSelectedObjects = TRUE
			.GetObjectRectangle nObjectIndex, nL, nT, nR, nB
			if RectanglesIntersect(nL, nT, nR, nB, nML, nMT, nMR, nMB) then
				nObjectsToSplit = nObjectsToSplit + 1
				if nObjectsToSplit > MAX_OBJECTS then
					MessageBox "Este guin puede dividir un mximo de" & Str(MAX_OBJECTS) & " objects at a time.", strDialogTitle, 0
					STOP
				endif
				anObjectIndices(nObjectsToSplit) = nObjectIndex
			endif
		endif
	next nObjectIndex

	if Not bFoundSelectedObjects then
		MessageBox "No hay objetos seleccionados", strDialogTitle, 0
		STOP
	endif

	if nObjectsToSplit <= 0 then
		MessageBox "No hay objetos seleccionados dentro del rectngulo delimitador de la mscara.", strDialogTitle, 0
		STOP
	endif

	REM	Display the Options dialog.

	MaintainOrder% = 1
	HideDoc% = 1
	nDialogResult = Dialog(SplitOptionsDialog)

	if nDialogResult = CANCELLED then
		STOP
	endif

	REM Hide the document to speed things up, if that option was selected.
	if HideDoc% = 1 then 
		.SetDocVisible FALSE
	endif

	REM Here's the (faster) version that doesn't maintain object order.

	if MaintainOrder% <> 1 then
		REM Select the objects and duplicate them en masse. The
		REM duplicates will be placed in front of the other objects.

		.ObjectSelectNone
		for nIndex = 1 to nObjectsToSplit
			.ObjectSelect anObjectIndices(nIndex), TRUE
		next nIndex
		.ObjectDuplicate
		nObjectCount = nObjectCount + nObjectsToSplit

		REM Correct the duplicate's positions.

		.ObjectSelectNone
		for nIndex = 1 to nObjectsToSplit
			.GetObjectRectangle anObjectIndices(nIndex), nLeft, nTop, nR, nB
			nCopyIndex = nObjectCount - nObjectsToSplit + nIndex
			.GetObjectRectangle nCopyIndex, nL, nT, nR, nB
			.ObjectSelect nCopyIndex, TRUE
			.ObjectTranslate nLeft - nL, nTop - nT
			.ObjectSelect nCopyIndex, FALSE
		next nIndex

		REM Select and clip the duplicates en masse.

		for nIndex = (nObjectCount - nObjectsToSplit + 1) to nObjectCount
			.ObjectSelect nIndex, TRUE
		next nIndex
		.ObjectClip

		REM Select and clip the original objects en masse.

		.ObjectSelectNone
		for nIndex = 1 to nObjectsToSplit
			.ObjectSelect anObjectIndices(nIndex), TRUE
		next nIndex
		.MaskInvert
		.ObjectClip
		.MaskInvert

		if HideDoc% = 1 then 
			.SetDocVisible TRUE
		endif

		REM All done.

		STOP
	endif

	REM Here's the version that maintains object ordering.

	REM Duplicate each selected object and use the object order commands
	REM to put the duplicate just ahead of the original.
	REM Then clip the original, taking care to account for objects that
	REM	are deleted by the clip.

	nIndexCorrection = 0
	.ObjectSelectNone
	.MaskInvert

	for nIndex = 1 to nObjectsToSplit
		anObjectIndices(nIndex) = anObjectIndices(nIndex) + nIndexCorrection
		.ObjectSelect anObjectIndices(nIndex), TRUE
		.GetObjectRectangle anObjectIndices(nIndex), nLeft, nTop, nR, nB
		.ObjectDuplicate
		nObjectCount = nObjectCount + 1
		nIndexCorrection = nIndexCorrection + 1
		.GetObjectRectangle nObjectCount, nL, nT, nR, nB
		.ObjectTranslate nLeft - nL, nTop - nT

		nDestIndex = anObjectIndices(nIndex) + 1

		if (nDestIndex - 1) < (nObjectCount - nDestIndex) then
			nNumberOfMoves = nDestIndex - 1
			nOrderChange = FORWARD
		else
			nNumberOfMoves = nObjectCount - nDestIndex
			nOrderChange = BACK
		endif

		if nNumberOfMoves > 0 then
			if nOrderChange = FORWARD then
				.ObjectOrder TO_BACK
			endif

			while nNumberOfMoves > 0
				.ObjectOrder nOrderChange
				nNumberOfMoves = nNumberOfMoves - 1
			wend
		endif

		REM Clip the original object.

		.ObjectSelect nDestIndex, FALSE
		.ObjectSelect anObjectIndices(nIndex), TRUE
		.ObjectClip

		REM Check whether the object was deleted by the clip.
		REM If it was, it won't be necessary to clip the copy
		REM to the inverse mask, so mark it with a negative index.

		if .GetObjectCount() < nObjectCount then
			nIndexCorrection = nIndexCorrection - 1
			nObjectCount = nObjectCount - 1
			anObjectIndices(nIndex) =  -1
		else
			.ObjectSelect anObjectIndices(nIndex), FALSE
		endif
	next nIndex

	REM Now clip all the duplicate objects in one step.

	.MaskInvert

	for nIndex = 1 to nObjectsToSplit
		if anObjectIndices(nIndex) >= 0 then
			.ObjectSelect anObjectIndices(nIndex) + 1, TRUE
		endif
	next nIndex

	.ObjectClip

	REM Deselect all objects, and show the document if it was hidden.

	.ObjectSelectNone

	if HideDoc% = 1 then 
		.SetDocVisible TRUE
	endif

	REM All done.

END WITHOBJECT



REM Function definitions.

Function RectanglesIntersect(nL1&, nT1&, nR1&, nB1&, nL2&, nT2&, nR2&, nB2&) as Boolean
	RectanglesIntersect = (nL1<=nR2) and (nR1>=nL2) and (nT1<=nB2) and (nB1>=nT2)
End Function

